#!/bin/sh

PERL="`which perl`"
EXPECTED_ARGS=6

if [ -z "$PERL" ] ; then
        echo "Program perl not found."
        exit 1
fi

echo "Found program:"
echo "${PERL}"
echo ""

if [ ! -f REAL-master/src/real ] ; then
        echo "Please run script install.sh to install REAL before running cREAL!!!"
        exit 1
fi

if [ $# -lt $EXPECTED_ARGS ] ; then
   echo " usage: sh creal.sh [options]"
   echo ""
   REAL-master/src/real
   exit 1
fi 

FOO=`/usr/bin/perl -x $0 $@ 2>&1`

echo <<'__END__' > /dev/null

#!/usr/bin/perl -wl
use strict;
use warnings;
use File::Find;
use File::Path;
use Cwd qw();
use File::Basename;

my @files   = ();
my @dirs    = ();
my @content = ();
my $dup_genome = 0;
my $dup_reads  = 0;
my $idx_genome = 0;
my $idx_reads  = 0;


# get the number of occurrences of switch -t
for ( my $i = 0; $i < @ARGV; ++ $i )
 {
   if ( $ARGV[$i] eq "-t" ) { $dup_genome ++; $idx_genome = $i; }
   if ( $ARGV[$i] eq "-p" ) { $dup_reads ++; $idx_reads = $i; }
 }

my $reads_file  = $ARGV[$idx_reads + 1];
my $genome_file = $ARGV[$idx_genome + 1];

# if it occurs more than once (or not at all) or does not have an argument
if ( $dup_genome != 1 or $dup_reads != 1 or $idx_genome + 1 >= @ARGV or $idx_reads + 1 >= @ARGV )
 {
   print "Error: incorrect -t or -p parameter usage!!!";
   exit;
 }

my $length = get_pattern_length()
  || die "Error: could not determine pattern length!!!";

#if the argument is a file place it in @files otherwise recursively scan the dir and group the files
unless ( -e $genome_file )
 {
   print "Error: $genome_file does not exist!!!";
   exit;
 }

push ( @files, $genome_file ) if ( -f $genome_file );
find ( sub { push ( @content, $File::Find::name ); }, $genome_file ) if ( -d $genome_file );
map { group_content ( $_ ); } @content;

mkdir ( "circular", 0777 ) or die $!;
map { my $dir = $_; mkdir ( "circular/" . $dir, 0777 ); } @dirs;

#copy all files
map { my $dest = basename($_); `cp $_ "circular/$dest"`; append_to_file ( "circular/$dest", $_ ); } @files;

$ARGV[$idx_genome + 1] = "circular" . "/" . basename($ARGV[$idx_genome + 1]);

print "@ARGV";

sub group_content
 {
   my ( $name ) = @_;

   push ( @files, $name ) if ( -f $name );
   push ( @dirs,  $name ) if ( -d $name );
 }

sub append_to_file 
 {
   my ( $copy, $src ) = @_;

   open (my $fh, '>>', $copy) || die "Error: could not open $copy: $!!!";
   print $fh _get_line ( $src ) or die "Error: could not print to $copy: $!!!";
   close $fh or die $!;
 }

sub get_pattern_length {
  open (my $fh, '<', $reads_file)
    || die "Error: could not open $reads_file for reading: $!!!";
  while(<$fh>) {
    chomp $_;
    $. == 1 && ( is_fasta_format($_)
                 || die "Error: $reads_file appears not to be in FASTA format!!!" );
    $_ =~ /^[ACGTN]*$/i || next;
    return length $_;
  }
  close $fh or die $!;
}

sub _get_line {
  my ( $genome_file ) = @_;
  open (my $fh, '<', $genome_file)
    || die "Error: could not open $genome_file for reading: $!!!";
  my $line = '';
  while(<$fh>) {
    chomp $_;
    $. == 1 && ( is_fasta_format($_)
                 || die "Error: $genome_file appears not to be in FASTA format!!!" );
    $_ =~ /^[ACGTN]*$/i || next;
    $line .= $_;
    last if $length < length $line;
  }
  $line = substr($line, 0, $length-1);
  close $fh or die $!;
  return $line
}

sub is_fasta_format {
  my $first_line = shift;
  return 1 if $first_line =~ /^\>.*$/;
  return 1 if $first_line =~ /^\@.*$/;
}

__END__

ERROR="Error"
echo $FOO | grep -q "$ERROR" && echo "$FOO" && echo "" && REAL-master/src/real && exit 1 || echo "Calling REAL with: $FOO" || REAL-master/src/real $FOO 2>&1 || rm -Rf circular
exit 1
